<?php
namespace Controller;

use Service\DeleteCascadeService;
use Service\ZmartLog as ZmarLogService;
use Service\Manager\Mongo\Manager;
use Zeedhi\Framework\DTO;
use Util\Utilities;
use Service\Traits\TResponse;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DataSource\FilterCriteria;

class ZmartLog extends Crud
{
    use TResponse;
    protected $dataSourceName = 'zmartLog';
    protected $DeleteCascadeService;
    protected $zmartLogService;

    /** @var \Service\Manager\Mongo\Manager */
    protected $organizationMongoManager;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, ZmarLogService $zmartLogService)
    {   $this->zmartLogService = $zmartLogService;
        $this->organizationMongoManager = $organizationMongoManager;
        $this->DeleteCascadeService = $deleteCascadeService;
        parent::__construct($organizationMongoManager, $deleteCascadeService);
        $this->mongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
    }

    public function findNext(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        Utilities::convertDynamicFilterToArray($params);
        Utilities::convertCreateFilterDateToMongoDateNext($params, "DATA_UPDATE");

        if(empty($params['order'])) {
           $params['order'] = ['DATA_UPDATE.desc'];
        }

        $dataSet = $this->zmartLogService->findNext($params, false);
        

         foreach ($dataSet as $key => &$row) {
               foreach ($row as &$data1) {
                   if ($data1 instanceof \DateTime){
                       $data1 = $data1->format('d/m/Y');
                   }
               }
               $dataSet[$key] = $row;
        }

        // $zmartLogService = Utilities::parseDataSourceZeedhiNext($dataSet);
        $data = Utilities::parseDataSourceZeedhiNext($dataSet, $this->count($params));
        $data["pagination"]["page"] = isset($params["page"]) ? (int)$params["page"] : 1;
        $data["pagination"]["limit"] = isset($params["limit"]) ? (int)$params["limit"] : 9999;
        $this->sendResponse($data, 200);
    }

    public function count($params) {
        return $this->mongoManager->countNext($this->dataSourceName, $params, false);
    }

     public function find(DTO\Request\Filter $request, DTO\Response $response)
    {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $sort = $this->mongoManager->processSort($filterData);
        if(empty($sort)){
            $sort = array("DATA_UPDATE" =>-1 );
        }
        $dataSet = $this->mongoManager->findByPagination($filterData,$sort);
        $response->addDataSet($dataSet);
        $dataSet = $response->getDataSets()[0];
        Utilities::datasetMongoToZeedhi($response, $dataSet,$this->dataSourceName);

    }



}
