import { Metadata, Http, DateHelper } from '@zeedhi/core';
import { Tab, Tabs, Form, Modal, ModalService, Select, DateRange} from '@zeedhi/common';
import { PeriodsService } from '@/Services/PeriodsService';
import { SkeletonMetadata } from '@/Services/SkeletonMetadata';
import { ZmartChartsApex } from '../Services/Chats/ZmartChartsApex';
import { ChartViewFactory } from '../Services/ChartView/ChartViewFactory';
import { Ordination } from '../Services/Ordination/Ordination';
import { Filter } from '../Services/Filter/Filter';

export class DashboardController {
    public tabsHeader: any = '';
    public dashboards: any = '';
    public widget: any = '';
    public countNameFormDashboard: any = 0;
    public nameFormDashboard: any = '';
    public headerTabs: any[] = [];
    public viewTypeId: any;
    public nameTypeChart: any;
    public dashboardCards: any;
    public dashboardId: any;
    public tabIndex: any = '';
    public countSkeleton: any = 0;
    public filterReloadDashboard: any = '';
    public getLastImportationDate: any = '';
    private filterDashboardModal!: Modal;
    periodFilterDashboard: any[] = [];
    structureLabelFilterDashboard: any = '';
    periodFilter: any[] = [];

    public async loadTabs() {
        this.filterReloadDashboard = '';
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        tab.tabs = [];
        const tabArray: any[] = [];
        const buildMainDashboards = await Filter.filterLoadTabs();
        this.tabsHeader = '';
        this.tabsHeader = await buildMainDashboards.data.dataset.dashboards;
        let num = 0;
        const skeletonMetada = new SkeletonMetadata();

        this.tabsHeader.forEach((tabs: any) => {
            const tabb = new Tab(
                {
                    "name": "tab" + num,
                    "tabTitle": tabs.name,
                    "teste": false,
                    "children": [
                        {
                            "name": "dashTab" + num,
                            "zmartSelected": false,
                            "zmartSelectedFilter": false,
                            "component": "ZdDashboard",
                            "isVisible": false,
                            "cards": []
                        },
                        SkeletonMetadata.skeleton(num)
                    ]
                }
            );
            tab.tabs.push(tabb);
            num += 1;
        });
        tab.activeTab = 0;
        this.tabIndex = tab.activeTab;
         if (tab.activeTab === 0 && this.tabsHeader[0]) {

            this.dashboards = await Filter.dashboardFilter(this.tabsHeader[0].name);

            tab.tabs[this.tabIndex].children[0].zmartSelected = true; // ZMARTSELECTED como true caso a TAB já tenha sido clicada (Visitada).
            this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, 0);
        }
    }

    public async clickTab({ tabIndex }: any) {
        this.tabIndex = tabIndex;
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');
        if (tab.tabs[tabIndex].children[0].zmartSelected === false) {

            this.dashboards = "";
            this.dashboards = await Filter.dashboardFilter(this.tabsHeader[tabIndex].name);

            if (this.periodFilterDashboard[0] !== undefined) {
                tab.tabs[tabIndex].children[0].cards = [];
                this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, tabIndex, true);
            } else {
                this.dashboardsTabs(this.dashboards.data.dataset.mainDashboard.indicators, tabIndex);
            }
        }
        tab.tabs[tabIndex].children[0].zmartSelected = true;
        tab.tabs[tabIndex].children[0].zmartSelectedFilter = true;
    }

    public async dashboardsTabs(dashboards: any, tabIndex: any, filter: boolean = false) {
        const tab = Metadata.getInstance<Tabs>('tabsDashboard');

        if (tab.tabs[this.tabIndex].children[0].zmartSelectedFilter === false) {
            const indicators: any[] = [];
            const dashboardIndicatorsIds: any[] = [];
            const tabName = Metadata.getInstance<Tabs>("tabsDashboard");
            tabName.tabs[tabIndex].children[0].cards = [];

            if (this.filterReloadDashboard !== '') {
                dashboards.forEach(async (dashboard: any) => {
                    dashboard.parameters.period.start = this.periodFilterDashboard[0];
                    dashboard.parameters.period.end = this.periodFilterDashboard[1];
                });
            }

            dashboards.forEach(function (dashboard: any) {
                if (!dashboardIndicatorsIds.includes(dashboard.indicator)) {
                    dashboardIndicatorsIds.push(dashboard.indicator);
                }
            });

            if (dashboards[0] !== undefined) {
                this.getLastImportationDate = "";
                this.getLastImportationDate = await Filter.filterGetLastImportationDates(dashboardIndicatorsIds, this.tabsHeader[tabIndex].name);
            }

            Ordination.sort(dashboards); // Ordenando os indicadores
            setTimeout(async () => {
                this.viewTypeId = {};
                this.dashboardCards = {};

            dashboards.forEach(async (dashboard: any) => {
                this.dashboardCards["override_" + dashboard._id] = dashboard;
                indicators.push(
                {
                    "cardId": "card" + dashboard._id,
                    "path": "dashboard/dashboards",
                    "local": true,
                    "overrideNamedProps": {
                        "NAME": "override_" + dashboard._id,
                    },
                    "height": this.heightCard(dashboard.height) + "px",
                    "width": dashboard.width
                }
                );
            });

                tabName.tabs[tabIndex].children[0].cards = indicators;
            }, 1000);
        }

        if (filter === true) {
            tab.tabs[tabIndex].children[0].zmartSelected = true;
            tab.tabs[tabIndex].children[0].zmartSelectedFilter = true;
        }

    }

    public async onMountedChildrenDashboard({component}: any) {
        const dashboard = this.dashboardCards[component.name];
        const tabName = Metadata.getInstance<Tabs>("tabsDashboard");

        const buildWidget = await Filter.filterBuildWidgetByViewIdNext(dashboard.parameters.period, dashboard._id);
        this.widget = '';
        this.widget = buildWidget.data.dataset.widget;
        this.viewTypeId[this.widget[0].id] = dashboard.viewTypeId;
        await ChartViewFactory.getChartType(this.viewTypeId[this.widget[0].id], dashboard, this.widget);
        const charts = ZmartChartsApex.build(this.widget, this, component);
        const widgetChats = charts.getCharts();
        widgetChats[0].grid = {
            "sm": "12",
            "lg": "12",
            "cols": "12",
            "md": "12",
        };
        widgetChats[0].height = this.heightCard(dashboard.height - 1) + "px";
        const componentDataImportation = this.periodLabel(dashboard);

        widgetChats.unshift(componentDataImportation); // Setando na primeira posição do Json

        if (this.widget[0].name === 'notFound') {
            component.children = this.widget;
            tabName.tabs[this.tabIndex].children[0].isVisible = true;
            tabName.tabs[this.tabIndex].children[1].isVisible = false;
        } else {
            component.children = widgetChats;
            tabName.tabs[this.tabIndex].children[0].isVisible = true;
            tabName.tabs[this.tabIndex].children[1].isVisible = false;
        }
    }

    public periodLabel(dashboard : any) {
        let componentDataImportation = {};
        const updateDate = this.getLastImportationDate.data.dataset.lastImportDates[dashboard.dataSourceId];
        if (dashboard.parameters.period.periodLabel === undefined) {
            componentDataImportation = {
                "name": "callDescriptionValue",
                "component": "ZdText",
                "cssClass": "zd-display-inline-flex",
                "cssStyle": "font-size: 11px; color: #aaa; line-height: 15px; font: Courier, monospace;",
                "text": dashboard.parameters.period.start + " - " + dashboard.parameters.period.end + " * Atualizado em : " + updateDate
              };
        } else {
            componentDataImportation = {
                "name": "callDescriptionValue",
                "component": "ZdText",
                "cssClass": "zd-display-inline-flex",
                "cssStyle": "font-size: 11px; color: #aaa; line-height: 15px; font: Courier, monospace;",
                "text": dashboard.parameters.period.periodLabel + " * Atualizado em : " + updateDate
              };
        }
        return componentDataImportation;
    }

    public heightCard(height: any) {
        return height * 130;
    }

    public instanceTab() {
        const tabs = Metadata.getInstance<Tabs>('tabsDashboard');

        tabs.tabs.forEach(function (tab: any) {
            tab.children[0].zmartSelectedFilter = false;
            tab.children[0].zmartSelected = false;
            tab.children[0].isVisible = false;
            tab.children[1].isVisible = true;
        });
    }

    public async reload() {
        this.instanceTab();
        const dashboards = await Filter.dashboardFilter(this.tabsHeader[this.tabIndex].name);
        const dashboardsIndicators = dashboards.data.dataset.mainDashboard.indicators;
        this.filterReloadDashboard = '';
        if (this.periodFilterDashboard[0] !== undefined) {
            this.filterReloadDashboard = dashboardsIndicators;
        }

        if (this.periodFilterDashboard[0] !== undefined) {
            dashboardsIndicators.forEach((dashboard: any) => {
                dashboard.parameters.period.start = this.periodFilterDashboard[0];
                dashboard.parameters.period.end = this.periodFilterDashboard[1];
            });
            this.filterReloadDashboard = dashboardsIndicators;
            this.dashboardsTabs(dashboardsIndicators, this.tabIndex, true);
        } else {
            this.dashboardsTabs(dashboardsIndicators, this.tabIndex);
        }
    }

    public filterDashboard() {
        this.createFilterDashboardModal();
    }

    public async createFilterDashboardModal() {
        this.filterDashboardModal = await ModalService.createFromJson('dashboard/modal/filterDashboard', true);
        this.filterDashboardModal.show();
    }

    public closeFiterDashboardModal() {
        this.filterDashboardModal.hide();
        this.filterDashboardModal?.destroy();
    }

    public async onMountedFilterDashboard() {
        // Se formulário não foi montado, não faz nada
        if (Metadata.getInstances('structureTypeForm').length === 0 || !Metadata.getInstances('structureTypeForm')[0]) {
            return;
        }
        const form = Metadata.getInstance<Form>('structureTypeForm');
        const structureLabel = Metadata.getInstance<Select>('structure_label');
        const period = form.getChildInstance<DateRange>('period');

        const periodDropdown = period.childrenInstances[0].childrenInstances[0];
        periodDropdown.maxHeight = 190;

        this.generatePeriodHelperOptions(period);
        const data = await Filter.filterGetStructureByUser();

        structureLabel.datasource.updateData(data.data.dataset.getStructureByUser);

        if (this.periodFilterDashboard[0] !== undefined) {
            form.value.period = this.periodFilter;
        }
    }

    public async generatePeriodHelperOptions(period: any) {
        const periodDate = await Http.post("/periods");
        const data = periodDate.data.data;
        if (period.helperOptions && period.helperOptions.length > 0) {
            period.helperOptions.forEach((option: any) => DateHelper.unregister(option.periodsCode));
        }
        data.forEach((option: any) => DateHelper.register(option.periodsCode, option.name, () => {
            const service = new PeriodsService();
            const periods = service.decodePeriod(option.periodsCode);
            return [periods.start, periods.end].map((date: any) => date.split('/').reverse().join('-')) as any;
        }));
        period.helperOptions = data.map((option: any) => option.periodsCode).filter((val: any) => val);
        period.helperValue = data.periodsCode;
        period.updateHelperHint();
    }

    public async saveFiterDashboard() {
        const form = Metadata.getInstance<Form>('structureTypeForm');
        let dataForm = false;
        if (!!form.value.period && !!form.value.period.length) {
            const dateStart = form.value.period[0];
            const dateEnd = form.value.period[1];
            dataForm = true;
            this.periodFilterDashboard = [dateStart, dateEnd];
            this.periodFilter = [dateStart, dateEnd];
        }
        if (dataForm === false) {
            this.periodFilterDashboard = [];
        }

        if (form.value.structure_label !== undefined) {
            this.structureLabelFilterDashboard = form.value.structure_label;
        }

        this.closeFiterDashboardModal();
        await this.reload();
    }
}
